/**********************************************************************
 *      gkrellm-gaim.c, communicate with a running gaim
 *
 *      This is a gkrellm gaim-plugin to communicate with a 
 *      gaim gkrellm-plugin and display gaim status and information
 *	inside a gkrellm panel.
 *
 *      (c) 2002, Cyril Bellot.
 */

#include <gkrellm2/gkrellm.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <errno.h>
#include <sys/un.h>

#include "status-all.xpm"

#define	CONFIG_NAME	"Gaim plugin"
#define	STYLE_NAME	"gaim"

#define NAMED_PIPE_FILE_NAME "/.gaim/gkrellm-gaim"
#define NAMED_PIPE_TRY_NB 5
static gint refresh=0;

static GkrellmTextstyle *ts, *ts_alt;

static GkrellmMonitor *monitor;
static GkrellmPanel   *panel;

static GkrellmDecal 		*text1_decal;
//static GkrellmDecal 		*text2_decal;
static GkrellmDecalbutton	*status_button;

static gchar    *scroll_text = "";
static gint	x_scroll=0;
static gint     scroll_text_index=0;

static gint     blinking=0;

static gint     style_id;

//static gint     button_state;
//static gchar    *button_text[2]	= {"Hello", "Bye" };

static FILE     *unix_pipe;


/**********************************************************************
 *    gaim_answered
 *    return to normal status
 */
static void gaim_answered() {
    blinking=0;

    /* remove scrolling text*/
    scroll_text=g_strdup("");

    /* switch to logged out icon */
    gkrellm_set_decal_button_index(status_button,0);
    gkrellm_draw_panel_layers(panel);
}

/**********************************************************************
 *    gaim_mesg_pending
 *    display the message pending status
 */
static void gaim_mesg_pending() {
    blinking=3;

    /* switch to first msg pending icon */
    gkrellm_set_decal_button_index(status_button,3);
    gkrellm_draw_panel_layers(panel);
}

/**********************************************************************
 *    icon_blinking
 *    blinks icon when a message is pending
 */
static void icon_blinking() {
    if (blinking) {
	if (blinking<5)
	    gkrellm_set_decal_button_index(status_button,2+blinking);
	else
	    gkrellm_set_decal_button_index(status_button,10-blinking);
	if (++blinking>6) blinking=1;

    }
}
/**********************************************************************
 *    pipe_init
 *    try to connect to gaim's named pipe to talk with him (her?)
 */
void pipe_init()
{   
    char *namedpipe_file;
    struct stat s;
    
    namedpipe_file=g_strconcat(getenv("HOME"), NAMED_PIPE_FILE_NAME, NULL);
    if (!stat(namedpipe_file, &s)) {
        unix_pipe=fopen(namedpipe_file,"r");
    } else {
        unix_pipe=NULL;
    }
    g_free(namedpipe_file);
}

/**********************************************************************
 *    pipe_read
 *    try to read through named pipe
 */
static void pipe_read(){
    guchar st1;
    fd_set rfds;
    struct timeval tv;
    gchar c[16];

    if (unix_pipe) {
        FD_ZERO(&rfds);
        FD_SET(fileno(unix_pipe), &rfds);
	tv.tv_sec = 0;
	tv.tv_usec = 10;
	if (select(fileno(unix_pipe)+1, &rfds, NULL, NULL, &tv)) {
	    /* if data is present read in pipe */
	    if (fread(&st1, sizeof(st1), 1, unix_pipe)>0) {
		switch (st1) {
		    case 8: gaim_mesg_pending();
			/* in case of msg pending, read also nickname */
			if (fread(c, sizeof(gchar), 16, unix_pipe)>0) {
			    scroll_text=g_strdup_printf("%s",c);	
			}
			break;
		    case 9: gaim_answered();
			break;
		}
	    }
	 }
    } else {
	/* try to reopen pipe only every NAMED_PIPE_TRY_NB times */
	if (refresh>gkrellm_update_HZ()*NAMED_PIPE_TRY_NB) {
		refresh=0;
		pipe_init();
	}
	refresh++;
    }
}

/**********************************************************************
 *    cb_button
 *    handles buttons clics
 */
static void cb_button(GkrellmDecalbutton *button, gpointer data) {

    if (GPOINTER_TO_INT(data)==0)
	gaim_answered();
/*    else if (GPOINTER_TO_INT(data)==1)
	printf("Small auto-hiding scaled button pressed\n");
    else if (GPOINTER_TO_INT(data)==2) {
	printf("button for text2_decal pressed\n");
	button_state=!button_state;

	gkrellm_draw_decal_text(panel, text2_decal,
				button_text[button_state], button_state);
	gkrellm_draw_panel_layers(panel);
    }
    else if (GPOINTER_TO_INT(data)==3)
	printf("button for pixmap_decal pressed\n");*/
}

/**********************************************************************
 *    panel_expose_event
 *    handles expose events
 */
static gint panel_expose_event(GtkWidget *widget, GdkEventExpose *ev) {
    gdk_draw_pixmap(widget->window,
	widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
	panel->pixmap, ev->area.x, ev->area.y, ev->area.x, ev->area.y,
	ev->area.width, ev->area.height);
    return FALSE;
}

/**********************************************************************
 *    update_plugin
 */
static void update_plugin() {
    static int w, t;

    /* scrolling text */
    t=gdk_string_width(ts->font, scroll_text);
    w=text1_decal->w;
    x_scroll=(x_scroll + 2) % (t+w);
    text1_decal->x_off=w-x_scroll;
    gkrellm_draw_decal_text(panel, text1_decal, scroll_text, scroll_text_index);
    scroll_text_index++;

    /* blinking icon */
    icon_blinking();

    /* read socket and gets status */
    pipe_read();

    /* draw panel */
    gkrellm_draw_panel_layers(panel);
}

/**********************************************************************
 *    create_plugin
 */
static void create_plugin(GtkWidget *vbox, gint first_create) {
    GkrellmPiximage	*pix;
    GkrellmStyle	*style;
    gint		y;
    gint		x;

    if (first_create) {
	panel=gkrellm_panel_new0();
	pipe_init();
    }

    style=gkrellm_meter_style(style_id);

    ts=gkrellm_meter_textstyle(style_id);
    ts_alt=gkrellm_meter_alt_textstyle(style_id);


    text1_decal=gkrellm_create_decal_text(panel, "Ay", ts, style, 15, -1, -1);

    y=text1_decal->y+text1_decal->h+1;

    pix=gkrellm_piximage_new_from_xpm_data(status_all_xpm);
    status_button=gkrellm_make_scaled_button(panel, pix, cb_button, NULL, FALSE,
    	    FALSE, 7, 0, 1, 2, 2, 12, 12);
    x=status_button->decal->x + status_button->decal->w + 2;

    /*
    text2_decal=gkrellm_create_decal_text(panel, "Hello", ts_alt, style,
	    x, y, 0);
    gkrellm_put_decal_in_meter_button(panel, text2_decal,
	    cb_button, GINT_TO_POINTER(2), NULL);
    gkrellm_draw_decal_text(panel, text2_decal, button_text[button_state],
	    button_state);
    */

    gkrellm_panel_configure(panel, NULL, style);
    gkrellm_panel_create(vbox, monitor, panel);

    if (first_create)
	g_signal_connect(G_OBJECT (panel->drawing_area), "expose_event",
	    G_CALLBACK(panel_expose_event), NULL);
}

/**********************************************************************
 *    GkrellmMonitor plugin_mon
 */ 
static GkrellmMonitor plugin_mon = {
	CONFIG_NAME,        /* Name, for config tab.    */
	0,                  /* Id,  0 if a plugin       */
	create_plugin,      /* The create function      */
	update_plugin,      /* The update function      */
	NULL,               /* The config tab create function   */
	NULL,               /* Apply the config function        */

	NULL,               /* Save user config   */
	NULL,               /* Load user config   */
	NULL,               /* config keyword     */

	NULL,               /* Undefined 2  */
	NULL,               /* Undefined 1  */
	NULL,               /* private      */

	MON_MAIL,           /* Insert plugin before this monitor */

	NULL,               /* Handle if a plugin, filled in by GKrellM     */
	NULL                /* path if a plugin, filled in by GKrellM       */
};

/**********************************************************************
 *    gkrellm_init_plugin
 */
GkrellmMonitor * gkrellm_init_plugin(void) {
    style_id=gkrellm_add_meter_style(&plugin_mon, STYLE_NAME);
    monitor=&plugin_mon;
    return &plugin_mon;
}
